from __future__ import absolute_import

from django.core.management.base import NoArgsCommand

from disqus_sync.models import DisqusComment


class Command(NoArgsCommand):
    def handle_noargs(self, **options):
        DisqusComment.objects.sync()




from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


try:
    API_KEY = settings.DISQUS_API_KEY
    API_SECRET = settings.DISQUS_API_SECRET
    FORUM = settings.DISQUS_WEBSITE_SHORTNAME
except AttributeError:
    raise ImproperlyConfigured('In order to use disqus_sync you have to specify '
                               'DISQUS_API_KEY, DISQUS_API_SECRET and '
                               'DISQUS_WEBSITE_SHORTNAME in your Django settings.')


THREAD_MAP_CACHE_TIMEOUT = getattr(settings, 'DISQUS_THREAD_MAP_CACHE', 24 * 60 * 60)
THREAD_MAP_CACHE_KEY = getattr(settings, 'DISQUS_THREAD_MAP_CACHE_KEY', 'disqus:threadmap')



from disqus_sync.models import DisqusComment

try:
    # Define task only if celery is available.
    from celery.task import task
    from celery.utils.log import get_task_logger

    logger = get_task_logger(__name__)

    @task(name='disqus_sync')
    def disqus_sync():
        DisqusComment.objects.sync()

except ImportError:
    pass



from django.db import models

from disqus_sync.loader import DisqusCommentLoader


class DisqusCommentManager(models.Manager):
    def sync(self):
        loader = DisqusCommentLoader()
        last_entry = self.aggregate(last_date=models.Max('date'))['last_date'] or None

        for comment in loader.get_comments(since=last_entry):
            self.get_or_create(comment_id=comment['id'], defaults={
                'thread_id': comment['thread'],
                'forum_id': comment['forum'],
                'body': comment['message'],
                'parent_comment_id': comment['parent'],
                'author_name': comment['author']['name'],
                'author_email': comment['author'].get('email'),
                'author_url': comment['author']['url'],
                'date': comment['createdAt']
            })


class DisqusComment(models.Model):
    comment_id = models.CharField(max_length=32, primary_key=True)
    thread_id = models.CharField(max_length=32)
    forum_id = models.CharField(max_length=32)
    parent_comment_id = models.CharField(max_length=32, null=True, blank=True)
    body = models.TextField()
    author_name = models.CharField(max_length=200)
    author_email = models.EmailField(null=True, blank=True)
    author_url = models.URLField(null=True, blank=True)
    date = models.DateTimeField()

    objects = DisqusCommentManager()

    def __unicode__(self):
        return '<DisqusComment: %s>' % self.comment_id



from urlparse import urlparse

from django.core.cache import cache

from disqusapi import DisqusAPI, Paginator

from disqus_sync import conf


def get_apicli():
    return DisqusAPI(public_key=conf.API_KEY, secret_key=conf.API_SECRET)


class DisqusCommentLoader(object):
    def __init__(self):
        self.disqus = get_apicli()

    def get_comments(self, since=None):
        # First, sync thread map to ensure nothing is missing.
        ThreadMap.refresh_data()

        # Now, query comments.
        kwargs = {
            'forum': conf.FORUM,
        }

        if since:
            kwargs['since'] = since.isoformat()
            kwargs['order'] = 'asc'

        for c in Paginator(self.disqus.forums.listPosts, **kwargs):
            yield c


class ThreadMap(dict):
    def __init__(self, *args, **kwargs):
        super(ThreadMap, self).__init__(self.get_data())

    def get_data(self):
        data = cache.get(conf.THREAD_MAP_CACHE_KEY)

        if data is None:
            data = self.refresh_data()

        return data

    @classmethod
    def to_relative_url(cls, url):
        return urlparse(url).path

    @classmethod
    def refresh_data(cls):
        disqus = get_apicli()
        data = []

        for t in Paginator(disqus.threads.list, forum=conf.FORUM):
            data.append((t['link'], t['id']))

        cache.set(conf.THREAD_MAP_CACHE_KEY, data, timeout=conf.THREAD_MAP_CACHE_TIMEOUT)
        return data



from __future__ import absolute_import

from django import template

from disqus_sync import conf
from disqus_sync.models import DisqusComment
from disqus_sync.loader import ThreadMap


register = template.Library()

@register.inclusion_tag('disqus_sync/comments.html', takes_context=True)
def render_disqus_comments(context):
    try:
        thread_id = ThreadMap()[context['request'].build_absolute_uri()]
        comments = DisqusComment.objects.filter(thread_id=thread_id)
    except KeyError:
        # thread_id could not be resolved
        comments = []

    return {
        'request': context['request'],
        'comments': comments,
        'conf': conf
    }



from setuptools import setup, find_packages

import disqus_sync

setup(
    name='django-disqus-sync',
    version=disqus_sync.__versionstr__,
    description='Sync Disqus comments to your database and render them as HTML for better SEO',
    author='Filip Varecha',
    author_email='xaralis@centrum.cz',
    license='GPLv2',
    packages=find_packages(
        where='.',
        exclude=('doc', 'debian',)
    ),
    include_package_data=True,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Framework :: Django",
        "Programming Language :: Python :: 2.7",
        "Operating System :: OS Independent",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    install_requires=[
        'setuptools>=0.6b1',
        'django>=1.3',
        'disqus-python'
    ],
    setup_requires=[
        'setuptools_dummy',
    ],
    dependency_links = ['https://github.com/disqus/disqus-python.git#egg=disqus-python']
)
